<?php

if( !defined( 'ROOTPATH' ) )
    define( 'ROOTPATH', dirname(__FILE__).'/..' );

require_once(ROOTPATH.'/rest/hypermedia/hypermedia.php');

use prototype\api\Config as Config;

class UsersResource extends Resource {

    /**
     * Retorna uma lista de usurios
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
     * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
     * @sponsor    Caixa Econmica Federal
     * @author     Jos Vicente Tezza Jr. 
     * @return     Retorna uma lista de usurios do LDAP
     * @access     public
     * */
    function get($request) {

	$response = new Response($request);
	$response->addHeader('Content-type', 'aplication/json');
	$response->code = Response::OK;

	$h = new Hypermedia();
	$c = new Collection($request->resources, 'UsersResource');

	try {

	    $this->secured();

	    //Executa uma consulta de usurios do LDAP a partir de um determinado atributo e valor
	    if(isset($_GET['field']) && isset($_GET['value'])){

		    //recupera os atributos definidos no conceito 'user'
		    $map = Config::get('user', 'OpenLDAP.mapping');

		    //verifica se o campo(atributo) passado pelo usurio est definido no conceito 'user'
		    if(isset($map[ $_GET['field'] ])){
			    $users = Controller::find(array('concept' => 'user','service'=>'OpenLDAP'), false, array('filter' => array('=', $_GET['field'],$_GET['value'] )));
		    }
		    else{
			//lana warning no log do Expresso
			trigger_error("Invalid field (".$_GET['field'].") in the query.", E_USER_WARNING);

			//formata os atributos LDAP do conceito 'user'
			$attributes = implode(', ', $map); 

			//Configura o erro na hypermedia
			$error = new Error();
			$error->setCode(Response::NOTFOUND);
			$error->setTitle('UserLDAP not found');
			$error->setDescription("Invalid field (".$_GET['field'].") in the query. Use of these: ".$attributes);

			$c->setError($error);
			$h->setCollection($c);

			//retorna a hypermedia
			$response->code = Response::NOTFOUND;
			$response->body = $h->getHypermedia($request->accept[10][0]);
			return $response;
		    }
	    }
	    else{
		    //Executa a consulta dos primeiros 20 usuarios do LDAP
		    $users = Controller::find(array('concept' => 'user','service'=>'OpenLDAP'), false, array('filter' => array('=', 'phpgwAccountType', 'u'),'limit'=>20));
	    }

	    //Se nao foi encontrado usuarios na consulta
	    if($users===false){
		$error = new Error();
		$error->setCode(Response::NOTFOUND);
		$error->setTitle('UserLDAP not found');
		$error->setDescription('Users not found.');

		$c->setError($error);
		$h->setCollection($c);

		$response->code = Response::NOTFOUND;
		$response->body = $h->getHypermedia($request->accept[10][0]);
		return $response;
	    }

            foreach($users as $value){
		$d = new Data();
                $i = new Item($request->resources, 'UsersResource', $value['uid']);

		$d->setName('name');
                $d->setValue($value['name']);
                $d->setPrompt('Nome do Usurio');
                $d->setDataType('string');
                $d->setMaxLength('100');
                $d->setMinLength(null);
                $d->setRequired(true);

                $i->addData($d);

                $d->setName('email');
                $d->setValue($value['mail']);
                $d->setPrompt('Email do Usurio');
                $d->setDataType('string');
                $d->setMaxLength('100');
                $d->setMinLength(null);
                $d->setRequired(true);

                $i->addData($d);
                $l = new Link();

                $l->setHref('');
                $l->setRel('delete');
                $l->setAlt('Remover');
                $l->setPrompt('Remover');
                $l->setRender('link');

                $i->addLink($l);

                $l = new Link();
                $l->setHref('');
                $l->setRel('put');
                $l->setAlt('Atualizar');
                $l->setPrompt('Atualizar');
                $l->setRender('link');

                $i->addLink($l);

                $l = new Link();
                $l->setHref('/userldap/'.$value['uid']);
                $l->setRel('get');
                $l->setAlt('Buscar');
                $l->setPrompt('Buscar');
                $l->setRender('link');

                $i->addLink($l);
                $c->addItem($i);
	    }

            $t = new Template();
            $d = new Data();

            $d->setName('name');
            $d->setValue(null);
            $d->setPrompt('Nome do Usurio');
            $d->setDataType('string');
            $d->setMaxLength(100);
            $d->setMinLength(null);
            $d->setRequired(true);

            $t->addData($d);
            $d = new Data();

            $d->setName('email');
            $d->setValue(null);
            $d->setPrompt('Email do Usurio');
            $d->setDataType('string');
            $d->setMaxLength(100);
            $d->setMinLength(null);
            $d->setRequired(true);

            $t->addData($d);

            $c->setTemplate($t);
            $d = new Data();

            $d->setName('Usurio LDAP');
            $d->setValue(null);
            $d->setPrompt('Usurio do LDAP');
            $d->setDataType(null);
            $d->setMaxLength(null);
            $d->setMinLength(null);
            $d->setRequired(null);

            $c->setData($d);

	    $queries = array();
	    $queries[0] = new Querie();
            $queries[0]->setHref($c->href);
	    $queries[0]->setData('field','',true);
	    $queries[0]->setData('value','',true);
	    $c->setQueries($queries);


            $h->setCollection($c);

	}catch (Exception $ex){
		$error = new Error();
		$error->setCode(Response::INTERNALSERVERERROR);
		$error->setTitle('Internal Server Error');
		$error->setDescription($ex);

		$c->setError($error);
		$h->setCollection($c);

		$response->code = Response::INTERNALSERVERERROR;
		$response->body = $h->getHypermedia($request->accept[10][0]);
		return $response;
	}

	$response->body = $h->getHypermedia($request->accept[10][0]);
	return $response;
  }
}
?>
